import os
import argparse
import sys
import logging

from blackduck import Client
from . import global_values

parser = argparse.ArgumentParser(description='Black Duck Signature component filter',
                                 prog='bd_sig_filter')

# parser.add_argument("projfolder", nargs="?", help="Yocto project folder to analyse", default=".")

parser.add_argument("--blackduck_url", type=str, help="Black Duck server URL (REQUIRED)", default="")
parser.add_argument("--blackduck_api_token", type=str, help="Black Duck API token (REQUIRED)", default="")
parser.add_argument("--blackduck_trust_cert", help="Black Duck trust server cert", action='store_true')
parser.add_argument("-p", "--project", help="Black Duck project to create (REQUIRED)", default="")
parser.add_argument("-v", "--version", help="Black Duck project version to create (REQUIRED)", default="")
parser.add_argument("--debug", help="Debug logging mode", action='store_true')
parser.add_argument("--logfile", help="Logging output file", default="")
parser.add_argument("--report_file", help="Report output file", default="")
parser.add_argument("--version_match_reqd", help="Component matches require version string in path", action='store_true')
parser.add_argument("--ignore", help="Ignore components in synopsys, default or test folders and duplicates with wrong version", action='store_true')
parser.add_argument("--review", help="Mark components reviewed", action='store_true')
parser.add_argument("--no_ignore_test", help="Do not ignore components in test folders", action='store_true')
parser.add_argument("--no_ignore_synopsys", help="Do not ignore components in synopsys tool folders", action='store_true')
parser.add_argument("--no_ignore_defaults", help="Do not ignore components in default folders", action='store_true')
parser.add_argument("--ignore_no_path_matches", help="Also ignore components with no component/version match in signature path", action='store_true')

args = parser.parse_args()

def check_args():
    terminate = False
    # if platform.system() != "Linux":
    #     print('''Please use this program on a Linux platform or extract data from a Yocto build then
    #     use the --bblayers_out option to scan on other platforms\nExiting''')
    #     sys.exit(2)
    if args.debug:
        global_values.debug = True
        loglevel = logging.DEBUG
    else:
        loglevel = logging.INFO

    if args.logfile != '':
        if os.path.exists(args.logfile):
            logging.error(f"Specified logfile '{args.logfile}' already exists - EXITING")
            sys.exit(2)
        logging.basicConfig(encoding='utf-8',
                            handlers=[logging.FileHandler(args.logfile), logging.StreamHandler(sys.stdout)],
                            level=loglevel)
    else:
        logging.basicConfig(level=loglevel)

    logging.info("ARGUMENTS:")
    for arg in vars(args):
        logging.info(f"--{arg}={getattr(args, arg)}")
    logging.info('')

    url = os.environ.get('BLACKDUCK_URL')
    if args.blackduck_url != '':
        global_values.bd_url = args.blackduck_url
    elif url is not None:
        global_values.bd_url = url
    else:
        logging.error("Black Duck URL not specified")
        terminate = True

    if args.project != "" and args.version != "":
        global_values.bd_project = args.project
        global_values.bd_version = args.version
    else:
        logging.error("Black Duck project/version not specified")
        terminate = True

    api = os.environ.get('BLACKDUCK_API_TOKEN')
    if args.blackduck_api_token != '':
        global_values.bd_api = args.blackduck_api_token
    elif api is not None:
        global_values.bd_api = api
    else:
        logging.error("Black Duck API Token not specified")
        terminate = True

    trustcert = os.environ.get('BLACKDUCK_TRUST_CERT')
    if trustcert == 'true' or args.blackduck_trust_cert:
        global_values.bd_trustcert = True

    if args.version_match_reqd:
        global_values.version_match_reqd = True

    if args.ignore:
        global_values.ignore = True

    if args.review:
        global_values.review = True

    if args.no_ignore_test:
        global_values.no_ignore_test = True

    if args.no_ignore_synopsys:
        global_values.no_ignore_synopsys = True

    if args.no_ignore_defaults:
        global_values.no_ignore_defaults = True

    if args.ignore_no_path_matches:
        if not args.ignore:
            logging.warning(f"Option --ignore_no_path_matches set without --ignore")
        global_values.ignore_no_path_matches = True

    if args.report_file:
        if os.path.exists(args.report_file):
            logging.error(f"Report file '{args.report_file}' already exists - exiting")
            terminate = True
        global_values.report_file = args.report_file


    if terminate:
        sys.exit(2)
    return


def connect():
    if global_values.bd_url == '':
        return None

    bd = Client(
        token=global_values.bd_api,
        base_url=global_values.bd_url,
        timeout=30,
        verify=global_values.bd_trustcert  # TLS certificate verification
    )
    try:
        bd.list_resources()
    except Exception as exc:
        logging.warning(f'Unable to connect to Black Duck server - {str(exc)}')
        return None

    logging.info(f'Connected to Black Duck server {global_values.bd_url}')
    return bd



#!/usr/bin/env python

# import argparse
# import json
import logging
import sys
# import os
# import requests
# import platform
# import asyncio
# import bd_data

from blackduck import Client
from . import global_values
# from ComponentListClass import ComponentList
# from ComponentClass import Component
# logging.basicConfig(level=logging.INFO)


def check_projver(bd, proj, ver):
	params = {
		'q': "name:" + proj,
		'sort': 'name',
	}

	projects = bd.get_resource('projects', params=params)
	for p in projects:
		if p['name'] == proj:
			versions = bd.get_resource('versions', parent=p, params=params)
			for v in versions:
				if v['versionName'] == ver:
					return v
			break
	else:
		logging.error(f"Version '{ver}' does not exist in project '{proj}'")
		sys.exit(2)

	logging.warning(f"Project '{proj}' does not exist")
	print('Available projects:')
	projects = bd.get_resource('projects')
	for proj in projects:
		print(proj['name'])
	sys.exit(2)


def get_all_projects(bd):
	projs = bd.get_resource('projects', items=True)

	projlist = []
	for proj in projs:
		projlist.append(proj['name'])
	return projlist


def get_bdproject(bdproj, bdver):
	global_values.bd = Client(
		token=global_values.bd_api,
		base_url=global_values.bd_url,
		verify=(not global_values.bd_trustcert),  # TLS certificate verification
		timeout=60
	)

	ver_dict = check_projver(global_values.bd, bdproj, bdver)
	return ver_dict



from .ComponentListClass import ComponentList
from .ComponentClass import Component
from . import bd_data
from . import bd_project
from . import global_values
from tabulate import tabulate
import logging

class BOM:
    def __init__(self):
        self.complist = ComponentList()
        logging.info(f"Working on project '{global_values.bd_project}' version '{global_values.bd_version}'")

        self.bdver_dict = bd_project.get_bdproject(global_values.bd_project, global_values.bd_version)

        res = global_values.bd.list_resources(self.bdver_dict)
        projver = res['href']
        thishref = f"{projver}/components?limit=1000"

        bom_arr = bd_data.get_paginated_data(thishref, "application/vnd.blackducksoftware.bill-of-materials-6+json")

        for comp in bom_arr:
            if 'componentVersion' not in comp:
                continue
            # compver = comp['componentVersion']

            compclass = Component(comp['componentName'], comp['componentVersionName'], comp)
            self.complist.add(compclass)

        return

    def get_bom_files(self):
        res = global_values.bd.list_resources(self.bdver_dict)
        projver = res['href']
        thishref = f"{projver}/source-bom-entries?filter=bomMatchType%3Afiles_modified&filter=bomMatchType%3Afiles_added_deleted&filter=bomMatchType%3Afile_exact&filter=bomMatchType%3Afiles_exact&limit=1000"
        thishref = thishref.replace("/api/", "/api/internal/")

        # https://poc39.blackduck.synopsys.com/api/internal/projects/9a25dee6-bc03-416c-a5bb-70ceb58ada28/versions/067cd508-ff40-46b8-ae0d-23218f224eeb/source-bom-entries?filter=bomMatchType%3Afiles_modified&filter=bomMatchType%3Afiles_added_deleted&filter=bomMatchType%3Afile_exact&filter=bomMatchType%3Afiles_exact&limit=100&offset=0

        # return bd_data.get_paginated_data(thishref, "application/vnd.blackducksoftware.internal-1+json")
        src_data = bd_data.get_paginated_data(thishref, "application/json")
        self.complist.add_bomfile_data(src_data)
        return

    def process(self):
        self.complist.process()
        return

    def update_components(self):
        self.complist.update_components(self.bdver_dict)

    def report_summary(self):
        table = [['Before', self.complist.count(), self.complist.count_ignored(), self.complist.count_reviewed(),
                  self.complist.count_not_reviewed_ignored()],
                 ['After', self.complist.count(), self.complist.count_to_be_ignored(),
                  self.complist.count_to_be_reviewed(), self.complist.count_not_to_be_reviewed_ignored()]]
        print("SUMMARY:")
        print(tabulate(table, headers=["", "Components", "Ignored", "Reviewed", "Neither"], tablefmt="simple"))
        print()

        if global_values.report_file != '':
            with open(global_values.report_file, "w") as rfile:
                # Writing data to a file
                rfile.writelines("SUMMARY:")
                rfile.writelines(tabulate(table, headers=["", "Components", "Ignored", "Reviewed", "Neither"],
                                          tablefmt="Simple"))
                rfile.writelines("")

    def report_full(self):
        table = self.complist.get_component_report_data()
        print(tabulate(table, headers=["Component", "Match Type", "Ignored", "Reviewed", "To be Ignored",
                                       "To be Reviewed", "Action"]))
        print()



# from Component import Component
from . import global_values
import logging
import requests

class ComponentList:
    components = []
    def __init__(self):
        pass

    def add(self, comp):
        self.components.append(comp)

    def add_comp_src_by_compverid(self, src_compverid, src_entry):
        for comp in self.components:
            compverid = comp.get_compverid()
            if compverid == src_compverid:
                comp.add_src(src_entry)
                return True
        return False

    def count(self):
        return len(self.components)

    def count_ignored(self):
        count = 0
        for comp in self.components:
            if comp.is_ignored():
                count += 1
        return count

    def count_reviewed(self):
        count = 0
        for comp in self.components:
            if comp.get_reviewed_status():
                count += 1
        return count

    def count_to_be_ignored(self):
        count = 0
        for comp in self.components:
            if comp.ignore:
                count += 1
        return count

    def count_to_be_reviewed(self):
        count = 0
        for comp in self.components:
            if comp.mark_reviewed:
                count += 1
        return count

    def count_not_reviewed_ignored(self):
        count = 0
        for comp in self.components:
            if not comp.is_ignored() and not comp.get_reviewed_status():
                count += 1
        return count

    def count_not_to_be_reviewed_ignored(self):
        count = 0
        for comp in self.components:
            if not comp.ignore and not comp.mark_reviewed:
                count += 1
        return count


    def add_bomfile_data(self, src_bomfile_arr):
        match_count = 0
        for src_entry in src_bomfile_arr:
            try:
                src_compverid = src_entry['fileMatchBomComponent']['release']['id']
                if self.add_comp_src_by_compverid(src_compverid, src_entry):
                    match_count += 1
            except KeyError:
                continue

        logging.debug(f"Elements added to comps {match_count}, Elements in src array {len(src_bomfile_arr)}")
        return

    def process(self):
        # For all components:
        # - if match_type contains Dependency then mark_reviewed
        # - else-if match_type is ONLY Signature then check_signature_rules
        #
        logging.debug("\nSIGNATURE SCAN FILTER PHASE")
        for comp in self.components:
            if comp.is_ignored():
                continue
            if comp.is_dependency():
                comp.set_reviewed()
                comp.reason = "Mark REVIEWED - Dependency"
            elif comp.is_only_signature():
                comp.process_signatures()

        # look for duplicate components (same compid) and ignore
        logging.debug("\nDUPLICATE SIGNATURE MATCHES FILTER PHASE")
        for i in range(len(self.components)):
            comp1 = self.components[i]
            if comp1.is_ignored() or comp1.is_dependency() or not comp1.is_only_signature() or comp1.ignore:
                continue

            for j in range(i + 1, len(self.components)):
                comp2 = self.components[j]
                if comp2.is_ignored() or comp2.is_dependency() or not comp2.is_only_signature() or comp2.ignore:
                    continue

                if comp1.get_compid() == comp2.get_compid():
                    if comp1.compname_found and not comp2.compname_found:
                        logging.debug(f"IGNORING {comp2.name}/{comp2.version} as it is a duplicate to {comp1.name}/{comp1.version}")
                        comp2.reason = f"Mark IGNORED - Is a duplicate of '{comp1.name[:25]}/{comp1.version[:10]}' but has no compname in Signature paths"
                        comp2.set_ignore()
                    elif not comp1.compname_found and comp2.compname_found:
                        logging.debug(f"IGNORING {comp1.name}/{comp1.version} as it is a duplicate to {comp2.name}/{comp2.version}")
                        comp1.set_ignore()
                        comp1.reason = f"Mark IGNORED - Is a duplicate to '{comp2.name[:25]}/{comp2.version[:10]}' but has no compname in Signature paths"
                    elif comp1.compver_found and not comp2.compver_found:
                        logging.debug(f"Will ignore {comp2.name}/{comp2.version} as it is a duplicate to {comp1.name}/{comp1.version} and path misses version")
                        comp2.set_ignore()
                        comp2.reason = f"Mark IGNORED - Is a duplicate to '{comp1.name[:25]}/{comp1.version[:10]}' but has no version in Signature paths"
                    elif not comp1.compver_found and comp2.compver_found:
                        logging.debug(f"Will ignore {comp1.name}/{comp1.version} as it is a duplicate to {comp2.name}/{comp2.version} and path misses version")
                        comp1.set_ignore()
                        comp1.reason = f"Mark IGNORED - Is a duplicate to '{comp2.name[:25]}/{comp2.version[:10]}' but has no version in Signature paths"
                    else:
                        # Nothing to do
                        logging.debug(f"- Will retain both components {comp1.filter_name}/{comp1.filter_version} and {comp2.filter_name}/{comp2.filter_version} - "
                              f"{comp1.sig_match_result},{comp2.sig_match_result}")

    def update_components(self, ver_dict):
        if global_values.ignore:
            logging.info("- Ignoring components ...")
            self.ignore_components(ver_dict)

        if global_values.review:
            logging.info("- Marking components reviewed ...")
            self.review_components(ver_dict)


    def ignore_components(self, ver_dict):
        count_ignored = 0
        ignore_array = []
        ignore_comps = []

        for comp in self.components:
            if comp.ignore:
                try:
                    ignore_array.append(comp.data['_meta']['href'])
                    count_ignored += 1
                except KeyError:
                    continue
                if count_ignored >= 99:
                    ignore_comps.append(ignore_array)
                    ignore_array = []
                    count_ignored = 0
        if len(ignore_array) > 0:
            ignore_comps.append(ignore_array)
        else:
            # Nothing to do
            return

        count = 0
        for ignore_array in ignore_comps:
            ignore_bulk_data = {
                "components": ignore_array,
                # "reviewStatus": "REVIEWED",
                "ignored": True,
                # "usage": "DYNAMICALLY_LINKED",
                # "inAttributionReport": true
            }

            comment_bulk_data = {
                "components": ignore_array,
                "comment": "Ignored by bd_sig_filter() script - Sig path in excluded/text folder or duplicate",
                # "ignored": True,
                # "usage": "DYNAMICALLY_LINKED",
                # "inAttributionReport": true
            }

            try:
                url = ver_dict['_meta']['href'] + '/bulk-adjustment'
                headers = {
                    "Accept": "application/vnd.blackducksoftware.bill-of-materials-6+json",
                    "Content-Type": "application/vnd.blackducksoftware.bill-of-materials-6+json"
                }
                r = global_values.bd.session.patch(url, json=ignore_bulk_data, headers=headers)
                r.raise_for_status()

                url = ver_dict['_meta']['href'] + '/bulk-comment'
                headers = {
                    "Accept": "application/vnd.blackducksoftware.bill-of-materials-6+json",
                    "Content-Type": "application/vnd.blackducksoftware.bill-of-materials-6+json"
                }
                r = global_values.bd.session.post(url, json=comment_bulk_data, headers=headers)
                r.raise_for_status()

                count += len(ignore_array)
            except requests.HTTPError as err:
                global_values.bd.http_error_handler(err)
        logging.info(f"- Ignored {count} components")

        return

    def review_components(self, ver_dict):
        count_reviewed = 0
        review_array = []
        review_comps = []

        for comp in self.components:
            if comp.mark_reviewed:
                try:
                    review_array.append(comp.data['_meta']['href'])
                    count_reviewed += 1
                except KeyError:
                    continue
                if count_reviewed >= 99:
                    review_comps.append(review_array)
                    review_array = []
                    count_reviewed = 0
        if len(review_array) > 0:
            review_comps.append(review_array)
        else:
            return

        count = 0
        for review_array in review_comps:
            review_bulk_data = {
                "components": review_array,
                "reviewStatus": "REVIEWED",
                # "ignored": True,
                # "usage": "DYNAMICALLY_LINKED",
                # "inAttributionReport": true
            }

            comment_bulk_data = {
                "components": review_array,
                "comment": "Reviewed by bd_sig_filter() script - comp name and/or version found in Sig path",
                # "ignored": True,
                # "usage": "DYNAMICALLY_LINKED",
                # "inAttributionReport": true
            }

            try:
                url = ver_dict['_meta']['href'] + '/bulk-adjustment'
                headers = {
                    "Accept": "application/vnd.blackducksoftware.bill-of-materials-6+json",
                    "Content-Type": "application/vnd.blackducksoftware.bill-of-materials-6+json"
                }
                r = global_values.bd.session.patch(url, json=review_bulk_data, headers=headers)
                r.raise_for_status()

                url = ver_dict['_meta']['href'] + '/bulk-comment'
                headers = {
                    "Accept": "application/vnd.blackducksoftware.bill-of-materials-6+json",
                    "Content-Type": "application/vnd.blackducksoftware.bill-of-materials-6+json"
                }
                r = global_values.bd.session.post(url, json=comment_bulk_data, headers=headers)
                r.raise_for_status()

                count += len(review_array)

            except requests.HTTPError as err:
                global_values.bd.http_error_handler(err)

        logging.info(f"- Marked {count} components as reviewed")

        return

    def get_component_report_data(self):
        data = []
        for comp in self.components:
            if comp.is_signature() and comp.is_dependency():
                type = "Dep+Sig"
            elif comp.is_dependency():
                type = "Dep"
            elif comp.is_only_signature():
                type = "Sig"
            else:
                type = "Other"
            data.append([f"{comp.name[:25]}/{comp.version[:10]}", type, comp.is_ignored(), comp.get_reviewed_status(),
                         comp.ignore, comp.mark_reviewed, comp.reason])
        return data




# import bd_data
# from ComponentList import ComponentList
# from Component import Component
from . import config
# import bd_project
import logging
from .BOMClass import BOM
# import platform


def main():
    config.check_args()

    bom = BOM()

    logging.debug('- Getting matched file data ... ')
    bom.get_bom_files()
    bom.process()
    bom.update_components()
    bom.report_summary()
    bom.report_full()

    return

if __name__ == '__main__':
    main()



from thefuzz import fuzz
import os
import re
from . import global_values
import logging

class SigEntry:
    def __init__(self, src_entry):
        try:
            self.src_entry = src_entry
            self.path = src_entry['commentPath']
            elements = re.split(r"!|#|" + os.sep, self.path)
            # self.elements = self.path.replace("!", os.sep).replace("#", os.sep).split(os.sep)
            self.elements = list(filter(None, elements))

        except KeyError:
            return

    def search_component(self, compname, compver):
        logging.debug("")
        logging.debug(f"search_component() Checking Comp '{compname}/{compver}' - {self.path}:")
        # If component_version_reqd:
        # - folder matches compname and compver
        # - folder1 matches compname and folder2 matches compver
        # Else:
        # - folder matches compname
        # Returns:
        # Bool1 - compname found
        # Bool2 - version found
        # Match_value - search result against both

        compstring = f"{compname} {compver}"
        element_in_compname = 0
        compver_in_element = 0

        # test of path search
        newpath = self.path.replace(os.sep, " ")
        newpath = re.sub(r"([a-zA-Z-]*)[0-9] ", "\1 ", newpath)
        comp_in_path = fuzz.token_set_ratio(compstring, newpath)
        logging.debug(f"search_component(): TEST comp_in_path is {comp_in_path}: path='{self.path}")

        found_compname_only = False
        for element in self.elements:
            pos = re.search(r"\.dll|\.obj|\.o|\.a|\.lib|\.iso|\.qcow2|\.vmdk|\.vdi|\.ova|\.nbi|\.vib|\.exe|\.img|"
                            "\.bin|\.apk|\.aac|\.ipa|\.msi|\.zip|\.gz|\.tar|\.xz|\.lz|\.bz2|\.7z|\.rar|"
                            "\.cpio|\.Z|\.lz4|\.lha|\.arj|\.jar|\.ear|\.war|\.rpm|\.deb|\.dmg|\.pki", element)
            if pos is not None:
                element = element[:pos.start()]
            # How much of the element string is from the compname and version?
            # - for example acl-1.3.0.jar
            # - Value of 100 indicates either compname or version exists in element
            element_in_compstring = fuzz.token_set_ratio(element, compstring)
            element_in_compname = fuzz.token_set_ratio(element, compname)
            compver_in_element = fuzz.token_set_ratio(compver, element)

            if element_in_compstring > 80:
                if compver_in_element > 50:
                    # element has both compname and version
                    logging.debug(f"search_component() - MATCHED component name & version ({compstring}) in '{element}'")
                    return True, True, element_in_compname + compver_in_element
                elif element_in_compname > 50 and len(element) > 2:
                    found_compname_only = True
                    logging.debug(f"search_component() - FOUND component name ONLY ({compname}) in '{element}'")
            elif found_compname_only:
                if compver_in_element > 50:
                    logging.debug(f"search_component() - MATCHED component version ({compver}) in '{element}'")
                    return True, True, element_in_compname + compver_in_element
                else:
                    test = 1

        if found_compname_only:
            logging.debug("search_component() - MATCHED Compname only")
            return True, False, element_in_compname + compver_in_element

        logging.debug(f"search_component() - NOT MATCHED")
        return False, False, 0


    def filter_folders(self):
        # Return True if path should be ignored + reason
        if not global_values.no_ignore_synopsys:
            syn_folders = ['.synopsys', 'synopsys-detect', '.coverity', 'synopsys-detect.jar',
                           'scan.cli.impl-standalone.jar', 'seeker-agent.tgz', 'seeker-agent.zip',
                           'Black_Duck_Scan_Installation']
            for e in self.elements:
                if e in syn_folders:
                    return True, f"Found '{e}' in Signature match path '{self.path}'"

        if not global_values.no_ignore_defaults:
            def_folders = ['.cache', '.m2', '.local', '.cache','.config', '.docker', '.npm', '.npmrc', '.pyenv',
                           '.Trash', '.git', 'node_modules']
            for e in self.elements:
                if e in def_folders:
                    return True, f"Found '{e}' in Signature match path '{self.path}'"

        if not global_values.no_ignore_test:
            test_folders = r"^test$|^tests$|^testsuite$"
            for e in self.elements:
                if re.search(test_folders, e, flags=re.IGNORECASE) is not None:
                    return True, f"Found '{e}' in Signature match path '{self.path}'"
                # if e in test_folders:
                #     return True, f"Found '{e}' in Signature match path '{self.path}'"

        return False, ''



# import aiohttp
# import asyncio
# import platform
# import logging

# import bd_data
from . import global_values


# async def async_main(comps, token):
#     async with aiohttp.ClientSession(trust_env=True) as session:
#         data_tasks = []
#         file_tasks = []
#
#         count = 0
#         for url, comp in comps.items():
#             count += 1
#
#             file_task = asyncio.ensure_future(async_get_files(session, comp, token))
#             file_tasks.append(file_task)
#
#         await asyncio.gather(*data_tasks)
#         all_files = dict(await asyncio.gather(*file_tasks))
#
#         await asyncio.sleep(0.250)
#         # print(f'- {count} components ')
#         #
#         # print(all_files)
#
#     return all_files


# async def async_get_files(session, comp, token):
#     filelist = []
#
#     if not global_values.bd_trustcert:
#         ssl = False
#     else:
#         ssl = None
#
#     # retfile = "NOASSERTION"
#     hrefs = comp['_meta']['links']
#
#     link = next((item for item in hrefs if item["rel"] == "matched-files"), None)
#     # link = next((item for item in hrefs if item["rel"] == "origins"), None)
#     if link:
#         thishref = link['href'] + '?limit=1000'
#         headers = {
#             'Authorization': f'Bearer {token}',
#             'accept': "application/vnd.blackducksoftware.bill-of-materials-6+json",
#             # 'accept': "application/vnd.blackducksoftware.component-detail-5+json",
#         }
#
#         # archive_ignore = False
#         async with session.get(thishref, headers=headers, ssl=ssl) as resp:
#             result_data = await resp.json()
#             for item in result_data['items']:
#                 filelist.append(item['filePath']['compositePathContext'])
#
#     return comp['componentVersion'], filelist
#
#
# def get_file_data(compdict):
#     if platform.system() == "Windows":
#         asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
#     file_dict = asyncio.run(async_main(compdict, global_values.bd.session.auth.bearer_token))
#
#     return file_dict


def get_paginated_data(url, accept_hdr):
    headers = {
        'accept': accept_hdr,
    }
    res = global_values.bd.get_json(url, headers=headers)
    if 'totalCount' in res and 'items' in res:
        total_comps = res['totalCount']
    else:
        return []

    ret_arr = []
    downloaded_comps = 0
    while downloaded_comps < total_comps:
        downloaded_comps += len(res['items'])

        ret_arr += res['items']

        newurl = f"{url}&offset={downloaded_comps}"
        res = global_values.bd.get_json(newurl, headers=headers)
        if 'totalCount' not in res or 'items' not in res:
            break

    return ret_arr




debug = False
bd = None
bd_project = ''
bd_version = ''
bd_api = ''
bd_url = ''
bd_trustcert = False

version_match_reqd = False
ignore = False
no_ignore_test = False
no_ignore_synopsys = False
no_ignore_defaults = False
ignore_no_path_matches = False
review = False
report_file = ''


from . import global_values
from .SigEntryClass import SigEntry
import re
# import global_values
import logging
# from thefuzz import fuzz

class Component:
    def __init__(self, name, version, data):
        self.name = name
        self.version = version
        self.data = data
        self.sigentry_arr = []
        self.ignore = False
        self.mark_reviewed = False
        self.filter_name = self.filter_name_string(name)
        self.filter_version = self.filter_version_string(version)
        self.sig_match_result = -1
        self.compname_found = False
        self.compver_found = False
        self.reason = 'No Action - compname or version not found in Sig paths'
        self.best_sigpath = ''

    def get_compverid(self):
        try:
            return self.data['componentVersion'].split('/')[-1]
        except KeyError:
            return ''

    def add_src(self, src_entry):
        sigentry = SigEntry(src_entry)
        self.sigentry_arr.append(sigentry)

    def get_matchtypes(self):
        try:
            return self.data['matchTypes']
        except KeyError:
            return []

    def is_dependency(self):
        dep_types = ['FILE_DEPENDENCY_DIRECT', 'FILE_DEPENDENCY_TRANSITIVE']
        match_types = self.get_matchtypes()
        for m in dep_types:
            if m in match_types:
                return True
        return False

    def is_signature(self):
        sig_types = ['FILE_EXACT', 'FILE_SOME_FILES_MODIFIED', 'FILE_FILES_ADDED_DELETED_AND_MODIFIED']
        match_types = self.get_matchtypes()
        for m in sig_types:
            if m in match_types:
                return True
        return False

    def is_only_signature(self):
        return (not self.is_dependency() and self.is_signature())

    def set_ignore(self):
        self.ignore = True

    def get_reviewed_status(self):
        try:
            if self.data['reviewStatus'] == 'REVIEWED':
                return True
        except KeyError:
            return False
        return False

    def set_reviewed(self):
        if not self.get_reviewed_status():
            self.mark_reviewed = True
        return

    def is_ignored(self):
        try:
            return self.data['ignored']
        except KeyError:
            return False

    def process_signatures(self):
        all_paths_ignoreable = True
        reason = ''
        for sigentry in self.sigentry_arr:
            ignore, reason = sigentry.filter_folders()
            if not ignore:
                all_paths_ignoreable = False
                break
            else:
                self.sigentry_arr.remove(sigentry)

        if all_paths_ignoreable:
            # Ignore
            reason = f"Mark IGNORED - {reason}"
            self.reason = reason
            logging.debug(f"- Component {self.filter_name}/{self.version}: {reason}")
            self.set_ignore()
        else:
        #     print(f"NOT Ignoring {self.name}/{self.version}")
            self.sig_match_result = 0
            set_reviewed = False
            ignore = True
            for sigentry in self.sigentry_arr:
                compname_found, compver_found,\
                    new_match_result = sigentry.search_component(self.filter_name, self.filter_version)
                logging.debug(f"Compname in path {compname_found}, Version in path {compver_found}, "
                              f"Match result {new_match_result}, Path '{sigentry.path}'")
                if compver_found:
                    self.compver_found = True
                    ignore = False
                if compname_found:
                    self.compname_found = True
                if global_values.version_match_reqd:
                    if compver_found:
                        set_reviewed = True
                        ignore = False
                elif compname_found:
                    set_reviewed = True
                    ignore = False
                if new_match_result > self.sig_match_result:
                    self.sig_match_result = new_match_result
                    self.best_sigpath = sigentry.path
                # print(self.name, self.version, src['commentPath'])
            if set_reviewed:
                if self.compver_found:
                    reason = f"Mark REVIEWED - Compname & version in path '{self.best_sigpath}', Match result {self.sig_match_result}"
                elif self.compname_found:
                    reason = f"Mark REVIEWED - Compname in path '{self.best_sigpath}', Match result {self.sig_match_result}"

                self.reason = reason
                logging.debug(f"- Component {self.name}/{self.version}: {reason}")
                self.set_reviewed()
            if ignore and global_values.ignore_no_path_matches:
                self.set_ignore()
                self.reason = f"Mark IGNORED - compname or version not found in paths & --ignore_no_path_matches set"

    @staticmethod
    def filter_name_string(name):
        # Remove common words
        # - for, with, in, on,
        # Remove strings in brackets
        # Replace / with space
        ret_name = re.sub(r"\(.*\)", r"", name)
        for rep in [r" for ", r" with ", r" in ", r" on ", r" a ", r" the ", r" by ",
                    r" and ", r"^apache | apache | apache$", r" bundle ", r" only | only$", r" from ",
                    r" to ", r" - "]:
            ret_name = re.sub(rep, " ", ret_name, flags=re.IGNORECASE)
        ret_name = re.sub(r"[/@#:]", " ", ret_name)
        ret_name = re.sub(r" \w$| \w |^\w ", r" ", ret_name)
        ret_name = ret_name.replace("::", " ")
        ret_name = re.sub(r"  *", r" ", ret_name)
        ret_name = re.sub(r"^ ", r"", ret_name)
        ret_name = re.sub(r" $", r"", ret_name)

        logging.debug(f"filter_name_string(): Compname '{name}' replaced with '{ret_name}'")
        return ret_name

    @staticmethod
    def filter_version_string(version):
        # Remove +git*
        # Remove -snapshot*
        # Replace / with space
        ret_version = re.sub(r"\+git.*", r"", version, re.IGNORECASE)
        ret_version = re.sub(r"-snapshot.*", r"", ret_version, re.IGNORECASE)
        ret_version = re.sub(r"/", r" ", ret_version)
        return ret_version

    def get_compid(self):
        try:
            compurl = self.data['component']
            return compurl.split('/')[-1]
        except KeyError:
            return ''
